use crate::parser::parser_context::ContextElement;
use crate::parser::parser_context::ContextTree;

use super::element::Element;
use super::error::Res;
use super::source::Source;

#[derive(Debug)]
pub struct Document<'s> {
    pub source: &'s str,
    pub zeroth_section: Option<Section<'s>>,
    pub children: Vec<Heading<'s>>,
}

#[derive(Debug)]
pub struct Heading<'s> {
    pub source: &'s str,
    pub children: Vec<DocumentElement<'s>>,
}

#[derive(Debug)]
pub struct Section<'s> {
    pub source: &'s str,
    pub children: Vec<Element<'s>>,
}

#[derive(Debug)]
pub enum DocumentElement<'s> {
    Heading(Heading<'s>),
    Section(Section<'s>),
}

impl<'s> Source<'s> for Document<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Source<'s> for DocumentElement<'s> {
    fn get_source(&'s self) -> &'s str {
        match self {
            DocumentElement::Heading(obj) => obj.source,
            DocumentElement::Section(obj) => obj.source,
        }
    }
}

pub fn document(input: &str) -> Res<&str, Document> {
    let initial_context: ContextTree<'_, '_> = ContextTree::new();
    let document_context =
        initial_context.with_additional_node(ContextElement::DocumentRoot(input));

    todo!()
}
